package javax.imageio;

import java.awt.Rectangle;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.image.RenderedImage;
import java.awt.image.Raster;
import java.util.ArrayList;
import java.awt.image.BufferedImage;
import java.util.Set;
import javax.imageio.metadata.IIOMetadata;
import java.util.Iterator;
import javax.imageio.stream.ImageInputStream;
import java.io.IOException;
import javax.imageio.event.IIOReadUpdateListener;
import javax.imageio.event.IIOReadProgressListener;
import javax.imageio.event.IIOReadWarningListener;
import java.util.List;
import java.util.Locale;
import javax.imageio.spi.ImageReaderSpi;

public abstract class ImageReader
{
    protected ImageReaderSpi originatingProvider;
    protected Object input;
    protected boolean seekForwardOnly;
    protected boolean ignoreMetadata;
    protected int minIndex;
    protected Locale[] availableLocales;
    protected Locale locale;
    protected List<IIOReadWarningListener> warningListeners;
    protected List<Locale> warningLocales;
    protected List<IIOReadProgressListener> progressListeners;
    protected List<IIOReadUpdateListener> updateListeners;
    private boolean abortFlag;
    
    protected ImageReader(ImageReaderSpi originatingProvider) {
        this.input = null;
        this.seekForwardOnly = false;
        this.ignoreMetadata = false;
        this.minIndex = 0;
        this.availableLocales = null;
        this.locale = null;
        this.warningListeners = null;
        this.warningLocales = null;
        this.progressListeners = null;
        this.updateListeners = null;
        this.abortFlag = false;
        this.originatingProvider = originatingProvider;
    }
    
    public String getFormatName() throws IOException {
        return this.originatingProvider.getFormatNames()[0];
    }
    
    public ImageReaderSpi getOriginatingProvider() {
        return this.originatingProvider;
    }
    
    public void setInput(Object input, boolean seekForwardOnly, boolean ignoreMetadata) {
        if (input != null) {
            boolean n = false;
            if (this.originatingProvider != null) {
                Class[] inputTypes = this.originatingProvider.getInputTypes();
                int i = 0;
                while (i < inputTypes.length) {
                    if (inputTypes[i].isInstance(input)) {
                        n = true;
                        break;
                    }
                    ++i;
                }
            }
            else if ((input instanceof ImageInputStream)) {
                n = true;
            }
            if (n == 0) {
                throw new IllegalArgumentException("Incorrect input type!");
            }
            this.seekForwardOnly = seekForwardOnly;
            this.ignoreMetadata = ignoreMetadata;
            this.minIndex = 0;
        }
        this.input = input;
    }
    
    public void setInput(Object o, boolean b) {
        this.setInput(o, b, false);
    }
    
    public void setInput(Object o) {
        this.setInput(o, false, false);
    }
    
    public Object getInput() {
        return this.input;
    }
    
    public boolean isSeekForwardOnly() {
        return this.seekForwardOnly;
    }
    
    public boolean isIgnoringMetadata() {
        return this.ignoreMetadata;
    }
    
    public int getMinIndex() {
        return this.minIndex;
    }
    
    public Locale[] getAvailableLocales() {
        if (this.availableLocales == null) {
            return null;
        }
        return (Locale[])(Locale[])this.availableLocales.clone();
    }
    
    public void setLocale(Locale locale) {
        if (locale != null) {
            Locale[] availableLocales = this.getAvailableLocales();
            boolean n = false;
            if (availableLocales != null) {
                int i = 0;
                while (i < availableLocales.length) {
                    if (locale.equals(availableLocales[i])) {
                        n = true;
                        break;
                    }
                    ++i;
                }
            }
            if (n == 0) {
                throw new IllegalArgumentException("Invalid locale!");
            }
        }
        this.locale = locale;
    }
    
    public Locale getLocale() {
        return this.locale;
    }
    
    public abstract int getNumImages(boolean p0) throws IOException;
    
    public abstract int getWidth(int p0) throws IOException;
    
    public abstract int getHeight(int p0) throws IOException;
    
    public boolean isRandomAccessEasy(int n) throws IOException {
        return false;
    }
    
    public float getAspectRatio(int n) throws IOException {
        return (float)this.getWidth(n) / (float)this.getHeight(n);
    }
    
    public ImageTypeSpecifier getRawImageType(int n) throws IOException {
        return (ImageTypeSpecifier)this.getImageTypes(n).next();
    }
    
    public abstract Iterator<ImageTypeSpecifier> getImageTypes(int p0) throws IOException;
    
    public ImageReadParam getDefaultReadParam() {
        return new ImageReadParam();
    }
    
    public abstract IIOMetadata getStreamMetadata() throws IOException;
    
    public IIOMetadata getStreamMetadata(String s, Set<String> set) throws IOException {
        return this.getMetadata(s, set, true, 0);
    }
    
    private IIOMetadata getMetadata(String s, Set set, boolean b, int n) throws IOException {
        if (s == null) {
            throw new IllegalArgumentException("formatName == null!");
        }
        if (set == null) {
            throw new IllegalArgumentException("nodeNames == null!");
        }
        IIOMetadata iioMetadata = b ? this.getStreamMetadata() : this.getImageMetadata(n);
        if (iioMetadata != null) {
            if (iioMetadata.isStandardMetadataFormatSupported() && s.equals("javax_imageio_1.0")) {
                return iioMetadata;
            }
            String nativeMetadataFormatName = iioMetadata.getNativeMetadataFormatName();
            if (nativeMetadataFormatName != null && s.equals(nativeMetadataFormatName)) {
                return iioMetadata;
            }
            String[] extraMetadataFormatNames = iioMetadata.getExtraMetadataFormatNames();
            if (extraMetadataFormatNames != null) {
                int i = 0;
                while (i < extraMetadataFormatNames.length) {
                    if (s.equals(extraMetadataFormatNames[i])) {
                        return iioMetadata;
                    }
                    ++i;
                }
            }
        }
        return null;
    }
    
    public abstract IIOMetadata getImageMetadata(int p0) throws IOException;
    
    public IIOMetadata getImageMetadata(int n, String s, Set<String> set) throws IOException {
        return this.getMetadata(s, set, false, n);
    }
    
    public BufferedImage read(int n) throws IOException {
        return this.read(n, null);
    }
    
    public abstract BufferedImage read(int p0, ImageReadParam p1) throws IOException;
    
    public IIOImage readAll(int n, ImageReadParam imageReadParam) throws IOException {
        if (n < this.getMinIndex()) {
            throw new IndexOutOfBoundsException("imageIndex < getMinIndex()!");
        }
        BufferedImage read = this.read(n, imageReadParam);
        ArrayList<BufferedImage> list = null;
        int numThumbnails = this.getNumThumbnails(n);
        if (numThumbnails > 0) {
            list = new ArrayList();
            for (int i = 0; i < numThumbnails; ++i) {
                list.add(this.readThumbnail(n, i));
            }
        }
        return new IIOImage(read, list, this.getImageMetadata(n));
    }
    
    public Iterator<IIOImage> readAll(Iterator<? extends ImageReadParam> iterator) throws IOException {
        ArrayList<IIOImage> list = new ArrayList();
        int minIndex = this.getMinIndex();
        this.processSequenceStarted(minIndex);
        while (true) {
            ImageReadParam imageReadParam = null;
            if (iterator != null && iterator.hasNext()) {
                Object next = iterator.next();
                if (next != null) {
                    if ((next instanceof ImageReadParam)) {
                        imageReadParam = (ImageReadParam)next;
                    }
                    else {
                        throw new IllegalArgumentException("Non-ImageReadParam supplied as part of params!");
                    }
                }
            }
            Label_0185: {
                BufferedImage read;
                try {
                    read = this.read(minIndex, imageReadParam);
                }
                catch (IndexOutOfBoundsException ex) {
                    break Label_0185;
                }
                Object o = null;
                int numThumbnails = this.getNumThumbnails(minIndex);
                if (numThumbnails > 0) {
                    o = new ArrayList();
                    for (int i = 0; i < numThumbnails; ++i) {
                        o.add(this.readThumbnail(minIndex, i));
                    }
                }
                list.add(new IIOImage(read, o, this.getImageMetadata(minIndex)));
                ++minIndex;
                continue;
            }
            this.processSequenceComplete();
            return list.iterator();
        }
    }
    
    public boolean canReadRaster() {
        return false;
    }
    
    public Raster readRaster(int n, ImageReadParam imageReadParam) throws IOException {
        throw new UnsupportedOperationException("readRaster not supported!");
    }
    
    public boolean isImageTiled(int n) throws IOException {
        return false;
    }
    
    public int getTileWidth(int n) throws IOException {
        return this.getWidth(n);
    }
    
    public int getTileHeight(int n) throws IOException {
        return this.getHeight(n);
    }
    
    public int getTileGridXOffset(int n) throws IOException {
        return 0;
    }
    
    public int getTileGridYOffset(int n) throws IOException {
        return 0;
    }
    
    public BufferedImage readTile(int n, int n2, int n3) throws IOException {
        if (n2 != 0 || n3 != 0) {
            throw new IllegalArgumentException("Invalid tile indices");
        }
        return this.read(n);
    }
    
    public Raster readTileRaster(int n, int n2, int n3) throws IOException {
        if (!this.canReadRaster()) {
            throw new UnsupportedOperationException("readTileRaster not supported!");
        }
        if (n2 != 0 || n3 != 0) {
            throw new IllegalArgumentException("Invalid tile indices");
        }
        return this.readRaster(n, null);
    }
    
    public RenderedImage readAsRenderedImage(int n, ImageReadParam imageReadParam) throws IOException {
        return this.read(n, imageReadParam);
    }
    
    public boolean readerSupportsThumbnails() {
        return false;
    }
    
    public boolean hasThumbnails(int n) throws IOException {
        return this.getNumThumbnails(n) > 0;
    }
    
    public int getNumThumbnails(int n) throws IOException {
        return 0;
    }
    
    public int getThumbnailWidth(int n, int n2) throws IOException {
        return this.readThumbnail(n, n2).getWidth();
    }
    
    public int getThumbnailHeight(int n, int n2) throws IOException {
        return this.readThumbnail(n, n2).getHeight();
    }
    
    public BufferedImage readThumbnail(int n, int n2) throws IOException {
        throw new UnsupportedOperationException("Thumbnails not supported!");
    }
    
    public synchronized void abort() {
        this.abortFlag = true;
    }
    
    protected synchronized boolean abortRequested() {
        return this.abortFlag;
    }
    
    protected synchronized void clearAbortRequest() {
        this.abortFlag = false;
    }
    
    static List addToList(List list, Object o) {
        if (list == null) {
            list = new ArrayList();
        }
        list.add(o);
        return list;
    }
    
    static List removeFromList(List list, Object o) {
        if (list == null) {
            return list;
        }
        list.remove(o);
        if (list.size() == 0) {
            list = null;
            return list;
        }
        return list;
    }
    
    public void addIIOReadWarningListener(IIOReadWarningListener iioReadWarningListener) {
        if (iioReadWarningListener == null) {
            return;
        }
        this.warningListeners = addToList(this.warningListeners, iioReadWarningListener);
        this.warningLocales = addToList(this.warningLocales, this.getLocale());
        return;
    }
    
    public void removeIIOReadWarningListener(IIOReadWarningListener iioReadWarningListener) {
        if (iioReadWarningListener == null || this.warningListeners == null) {
            return;
        }
        int index = this.warningListeners.indexOf(iioReadWarningListener);
        if (index != -1) {
            this.warningListeners.remove(index);
            this.warningLocales.remove(index);
            if (this.warningListeners.size() == 0) {
                this.warningListeners = null;
                this.warningLocales = null;
            }
        }
        return;
    }
    
    public void removeAllIIOReadWarningListeners() {
        this.warningListeners = null;
        this.warningLocales = null;
    }
    
    public void addIIOReadProgressListener(IIOReadProgressListener iioReadProgressListener) {
        if (iioReadProgressListener == null) {
            return;
        }
        this.progressListeners = addToList(this.progressListeners, iioReadProgressListener);
        return;
    }
    
    public void removeIIOReadProgressListener(IIOReadProgressListener iioReadProgressListener) {
        if (iioReadProgressListener == null || this.progressListeners == null) {
            return;
        }
        this.progressListeners = removeFromList(this.progressListeners, iioReadProgressListener);
        return;
    }
    
    public void removeAllIIOReadProgressListeners() {
        this.progressListeners = null;
    }
    
    public void addIIOReadUpdateListener(IIOReadUpdateListener iioReadUpdateListener) {
        if (iioReadUpdateListener == null) {
            return;
        }
        this.updateListeners = addToList(this.updateListeners, iioReadUpdateListener);
        return;
    }
    
    public void removeIIOReadUpdateListener(IIOReadUpdateListener iioReadUpdateListener) {
        if (iioReadUpdateListener == null || this.updateListeners == null) {
            return;
        }
        this.updateListeners = removeFromList(this.updateListeners, iioReadUpdateListener);
        return;
    }
    
    public void removeAllIIOReadUpdateListeners() {
        this.updateListeners = null;
    }
    
    protected void processSequenceStarted(int n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).sequenceStarted(this, n);
        }
        return;
    }
    
    protected void processSequenceComplete() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).sequenceComplete(this);
        }
        return;
    }
    
    protected void processImageStarted(int n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).imageStarted(this, n);
        }
        return;
    }
    
    protected void processImageProgress(float n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).imageProgress(this, n);
        }
        return;
    }
    
    protected void processImageComplete() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).imageComplete(this);
        }
        return;
    }
    
    protected void processThumbnailStarted(int n, int n2) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).thumbnailStarted(this, n, n2);
        }
        return;
    }
    
    protected void processThumbnailProgress(float n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).thumbnailProgress(this, n);
        }
        return;
    }
    
    protected void processThumbnailComplete() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).thumbnailComplete(this);
        }
        return;
    }
    
    protected void processReadAborted() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadProgressListener)this.progressListeners.get(i)).readAborted(this);
        }
        return;
    }
    
    protected void processPassStarted(BufferedImage bufferedImage, int n, int n2, int n3, int n4, int n5, int n6, int n7, int[] array) {
        if (this.updateListeners == null) {
            return;
        }
        for (int size = this.updateListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadUpdateListener)this.updateListeners.get(i)).passStarted(this, bufferedImage, n, n2, n3, n4, n5, n6, n7, array);
        }
        return;
    }
    
    protected void processImageUpdate(BufferedImage bufferedImage, int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        if (this.updateListeners == null) {
            return;
        }
        for (int size = this.updateListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadUpdateListener)this.updateListeners.get(i)).imageUpdate(this, bufferedImage, n, n2, n3, n4, n5, n6, array);
        }
        return;
    }
    
    protected void processPassComplete(BufferedImage bufferedImage) {
        if (this.updateListeners == null) {
            return;
        }
        for (int size = this.updateListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadUpdateListener)this.updateListeners.get(i)).passComplete(this, bufferedImage);
        }
        return;
    }
    
    protected void processThumbnailPassStarted(BufferedImage bufferedImage, int n, int n2, int n3, int n4, int n5, int n6, int n7, int[] array) {
        if (this.updateListeners == null) {
            return;
        }
        for (int size = this.updateListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadUpdateListener)this.updateListeners.get(i)).thumbnailPassStarted(this, bufferedImage, n, n2, n3, n4, n5, n6, n7, array);
        }
        return;
    }
    
    protected void processThumbnailUpdate(BufferedImage bufferedImage, int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        if (this.updateListeners == null) {
            return;
        }
        for (int size = this.updateListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadUpdateListener)this.updateListeners.get(i)).thumbnailUpdate(this, bufferedImage, n, n2, n3, n4, n5, n6, array);
        }
        return;
    }
    
    protected void processThumbnailPassComplete(BufferedImage bufferedImage) {
        if (this.updateListeners == null) {
            return;
        }
        for (int size = this.updateListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadUpdateListener)this.updateListeners.get(i)).thumbnailPassComplete(this, bufferedImage);
        }
        return;
    }
    
    protected void processWarningOccurred(String s) {
        if (this.warningListeners == null) {
            return;
        }
        if (s == null) {
            throw new IllegalArgumentException("warning == null!");
        }
        for (int size = this.warningListeners.size(), i = 0; i < size; ++i) {
            ((IIOReadWarningListener)this.warningListeners.get(i)).warningOccurred(this, s);
        }
        return;
    }
    
    protected void processWarningOccurred(String s, String key) {
        if (this.warningListeners == null) {
            return;
        }
        if (s == null) {
            throw new IllegalArgumentException("baseName == null!");
        }
        if (key == null) {
            throw new IllegalArgumentException("keyword == null!");
        }
        for (int size = this.warningListeners.size(), i = 0; i < size; ++i) {
            IIOReadWarningListener iioReadWarningListener = (IIOReadWarningListener)this.warningListeners.get(i);
            Locale default1 = (Locale)this.warningLocales.get(i);
            if (default1 == null) {
                default1 = Locale.getDefault();
            }
            ClassLoader loader = (ClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    return Thread.currentThread().getContextClassLoader();
                }
            });
            ResourceBundle resourceBundle;
            try {
                resourceBundle = ResourceBundle.getBundle(s, default1, loader);
            }
            catch (MissingResourceException ex) {
                try {
                    resourceBundle = ResourceBundle.getBundle(s, default1);
                }
                catch (MissingResourceException ex2) {
                    throw new IllegalArgumentException("Bundle not found!");
                }
            }
            String string;
            try {
                string = resourceBundle.getString(key);
            }
            catch (ClassCastException ex3) {
                throw new IllegalArgumentException("Resource is not a String!");
            }
            catch (MissingResourceException ex4) {
                throw new IllegalArgumentException("Resource is missing!");
            }
            iioReadWarningListener.warningOccurred(this, string);
        }
        return;
    }
    
    public void reset() {
        this.setInput(null, false, false);
        this.setLocale(null);
        this.removeAllIIOReadUpdateListeners();
        this.removeAllIIOReadProgressListeners();
        this.removeAllIIOReadWarningListeners();
        this.clearAbortRequest();
    }
    
    public void dispose() {
    }
    
    protected static Rectangle getSourceRegion(ImageReadParam imageReadParam, int width, int height) {
        Rectangle intersection = new Rectangle(0, 0, width, height);
        if (imageReadParam != null) {
            Rectangle sourceRegion = imageReadParam.getSourceRegion();
            if (sourceRegion != null) {
                intersection = intersection.intersection(sourceRegion);
            }
            int subsamplingXOffset = imageReadParam.getSubsamplingXOffset();
            int subsamplingYOffset = imageReadParam.getSubsamplingYOffset();
            intersection.x += subsamplingXOffset;
            intersection.y += subsamplingYOffset;
            intersection.width -= subsamplingXOffset;
            intersection.height -= subsamplingYOffset;
            return intersection;
        }
        else {
            return intersection;
        }
    }
    
    protected static void computeRegions(ImageReadParam imageReadParam, int n, int n2, BufferedImage bufferedImage, Rectangle rectangle, Rectangle rectangle2) {
        if (rectangle == null) {
            throw new IllegalArgumentException("srcRegion == null!");
        }
        if (rectangle2 == null) {
            throw new IllegalArgumentException("destRegion == null!");
        }
        rectangle.setBounds(0, 0, n, n2);
        rectangle2.setBounds(0, 0, n, n2);
        int sourceXSubsampling = 1;
        int sourceYSubsampling = 1;
        if (imageReadParam != null) {
            Rectangle sourceRegion = imageReadParam.getSourceRegion();
            if (sourceRegion != null) {
                rectangle.setBounds(rectangle.intersection(sourceRegion));
            }
            sourceXSubsampling = imageReadParam.getSourceXSubsampling();
            sourceYSubsampling = imageReadParam.getSourceYSubsampling();
            int subsamplingXOffset = imageReadParam.getSubsamplingXOffset();
            int subsamplingYOffset = imageReadParam.getSubsamplingYOffset();
            rectangle.translate(subsamplingXOffset, subsamplingYOffset);
            rectangle.width -= subsamplingXOffset;
            rectangle.height -= subsamplingYOffset;
            rectangle2.setLocation(imageReadParam.getDestinationOffset());
        }
        if (rectangle2.x < 0) {
            int n3 = -rectangle2.x * sourceXSubsampling;
            rectangle.x += n3;
            rectangle.width -= n3;
            rectangle2.x = 0;
        }
        if (rectangle2.y < 0) {
            int n4 = -rectangle2.y * sourceYSubsampling;
            rectangle.y += n4;
            rectangle.height -= n4;
            rectangle2.y = 0;
        }
        int width = (rectangle.width + sourceXSubsampling - 1) / sourceXSubsampling;
        int height = (rectangle.height + sourceYSubsampling - 1) / sourceYSubsampling;
        rectangle2.width = width;
        rectangle2.height = height;
        if (bufferedImage != null) {
            rectangle2.setBounds(rectangle2.intersection(new Rectangle(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight())));
            if (rectangle2.isEmpty()) {
                throw new IllegalArgumentException("Empty destination region!");
            }
            int n5 = rectangle2.x + width - bufferedImage.getWidth();
            if (n5 > 0) {
                rectangle.width -= n5 * sourceXSubsampling;
            }
            int n6 = rectangle2.y + height - bufferedImage.getHeight();
            if (n6 > 0) {
                rectangle.height -= n6 * sourceYSubsampling;
            }
        }
        if (rectangle.isEmpty() || rectangle2.isEmpty()) {
            throw new IllegalArgumentException("Empty region!");
        }
        return;
    }
    
    protected static void checkReadParamBandSettings(ImageReadParam imageReadParam, int n, int n2) {
        int[] sourceBands = null;
        int[] destinationBands = null;
        if (imageReadParam != null) {
            sourceBands = imageReadParam.getSourceBands();
            destinationBands = imageReadParam.getDestinationBands();
        }
        if (((sourceBands == null) ? n : sourceBands.length) != ((destinationBands == null) ? n2 : destinationBands.length)) {
            throw new IllegalArgumentException("ImageReadParam num source & dest bands differ!");
        }
        if (sourceBands != null) {
            int i = 0;
            while (i < sourceBands.length) {
                if (sourceBands[i] >= n) {
                    throw new IllegalArgumentException("ImageReadParam source bands contains a value >= the number of source bands!");
                }
                ++i;
            }
        }
        if (destinationBands != null) {
            int j = 0;
            while (j < destinationBands.length) {
                if (destinationBands[j] >= n2) {
                    throw new IllegalArgumentException("ImageReadParam dest bands contains a value >= the number of dest bands!");
                }
                ++j;
            }
        }
        return;
    }
    
    protected static BufferedImage getDestination(ImageReadParam imageReadParam, Iterator<ImageTypeSpecifier> iterator, int n, int n2) throws IIOException {
        if (iterator == null || !iterator.hasNext()) {
            throw new IllegalArgumentException("imageTypes null or empty!");
        }
        if ((long)n * (long)n2 > 2147483647L) {
            throw new IllegalArgumentException("width*height > Integer.MAX_VALUE!");
        }
        ImageTypeSpecifier destinationType = null;
        if (imageReadParam != null) {
            BufferedImage destination = imageReadParam.getDestination();
            if (destination != null) {
                return destination;
            }
            destinationType = imageReadParam.getDestinationType();
        }
        if (destinationType == null) {
            Object next = iterator.next();
            if (!(next instanceof ImageTypeSpecifier)) {
                throw new IllegalArgumentException("Non-ImageTypeSpecifier retrieved from imageTypes!");
            }
            destinationType = (ImageTypeSpecifier)next;
        }
        else {
            boolean n3 = false;
            while (iterator.hasNext()) {
                if (((ImageTypeSpecifier)iterator.next()).equals(destinationType)) {
                    n3 = true;
                    break;
                }
                continue;
            }
            if (n3 == 0) {
                throw new IIOException("Destination type from ImageReadParam does not match!");
            }
        }
        Rectangle o = new Rectangle(0, 0, 0, 0);
        Rectangle o2 = new Rectangle(0, 0, 0, 0);
        computeRegions(imageReadParam, n, n2, null, o, o2);
        return destinationType.createBufferedImage(o2.x + o2.width, o2.y + o2.height);
    }
}
